/**
 * Copyright (c) 2014-present, Facebook, Inc. All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 *
 * 
 */

'use strict';





const ansiEscapes = require('ansi-escapes');
const chalk = require('chalk');var _require =
require('./lib/terminalUtils');const getTerminalWidth = _require.getTerminalWidth;
const highlight = require('./lib/highlight');
const stringLength = require('string-length');var _require2 =
require('./reporters/utils');const trimAndFormatPath = _require2.trimAndFormatPath;
const Prompt = require('./lib/Prompt');






const pluralizeFile = total => total === 1 ? 'file' : 'files';

const usage = () =>
`\n${chalk.bold('Pattern Mode Usage')}\n` +
` ${chalk.dim('\u203A Press')} Esc ${chalk.dim('to exit pattern mode.')}\n` +
` ${chalk.dim('\u203A Press')} Enter ` +
`${chalk.dim('to apply pattern to all filenames.')}\n` +
`\n`;

const usageRows = usage().split('\n').length;

module.exports = class TestPathPatternPrompt {





  constructor(pipe, prompt) {
    this._pipe = pipe;
    this._prompt = prompt;
    this._currentUsageRows = usageRows;
  }

  run(onSuccess, onCancel, options) {
    this._pipe.write(ansiEscapes.cursorHide);
    this._pipe.write(ansiEscapes.clearScreen);
    if (options && options.header) {
      this._pipe.write(options.header + '\n');
      this._currentUsageRows = usageRows + options.header.split('\n').length;
    } else {
      this._currentUsageRows = usageRows;
    }
    this._pipe.write(usage());
    this._pipe.write(ansiEscapes.cursorShow);

    this._prompt.enter(this._onChange.bind(this), onSuccess, onCancel);
  }

  _onChange(pattern) {
    let regex;

    try {
      regex = new RegExp(pattern, 'i');
    } catch (e) {}

    let tests = [];
    if (regex) {
      this._searchSources.forEach((_ref) => {let searchSource = _ref.searchSource,context = _ref.context;
        tests = tests.concat(searchSource.findMatchingTests(pattern).tests);
      });
    }

    this._pipe.write(ansiEscapes.eraseLine);
    this._pipe.write(ansiEscapes.cursorLeft);
    this._printTypeahead(pattern, tests, 10);
  }

  _printTypeahead(pattern, allResults, max) {
    const total = allResults.length;
    const results = allResults.slice(0, max);
    const inputText = `${chalk.dim(' pattern \u203A')} ${pattern}`;

    this._pipe.write(ansiEscapes.eraseDown);
    this._pipe.write(inputText);
    this._pipe.write(ansiEscapes.cursorSavePosition);

    if (pattern) {
      if (total) {
        this._pipe.write(
        `\n\n Pattern matches ${total} ${pluralizeFile(total)}.`);

      } else {
        this._pipe.write(`\n\n Pattern matches no files.`);
      }

      const width = getTerminalWidth();
      const prefix = `  ${chalk.dim('\u203A')} `;
      const padding = stringLength(prefix) + 2;

      results.
      map((_ref2) => {let path = _ref2.path,context = _ref2.context;
        const filePath = trimAndFormatPath(
        padding,
        context.config,
        path,
        width);

        return highlight(path, filePath, pattern, context.config.rootDir);
      }).
      forEach(filePath =>
      this._pipe.write(`\n  ${chalk.dim('\u203A')} ${filePath}`));


      if (total > max) {
        const more = total - max;
        this._pipe.write(
        // eslint-disable-next-line max-len
        `\n  ${chalk.dim(`\u203A and ${more} more ${pluralizeFile(more)}`)}`);

      }
    } else {
      this._pipe.write(
      // eslint-disable-next-line max-len
      `\n\n ${chalk.italic.yellow('Start typing to filter by a filename regex pattern.')}`);

    }

    this._pipe.write(
    ansiEscapes.cursorTo(stringLength(inputText), this._currentUsageRows - 1));

    this._pipe.write(ansiEscapes.cursorRestorePosition);
  }

  updateSearchSources(searchSources) {
    this._searchSources = searchSources;
  }};